import os
from dataclasses import dataclass
from typing import Tuple

from rich import get_console
from rich.prompt import Prompt
from rich.status import Status

from localci.cli_utils import call
from localci.git_utils import tag_exists, changes_exist


@dataclass
class Platform:
    project_name: str
    project_version: str

    username: str
    password: str

    def __init__(self):
        self.console = get_console()
        self.project_name, self.project_version = self.get_project_info()

        self.check_if_valid()
        self.confirmation_dialog()
        self.login_dialog()

    @staticmethod
    def get_registry_url() -> str:
        return os.getenv('LOCALCI_REGISTRY_URL', 'registry.sirgje.space')

    def login_dialog(self) -> None:
        pass

    def confirmation_dialog(self) -> None:
        if not self.project_name or not self.project_version:
            self.console.print('Could not read project information from this directory.', style='bold red')
            exit()
        
        # Confirmation dialog for build process
        confirmation = Prompt.ask(f'Building project: {self.application_name}',
                                  choices=['Y', 'N'])
        if confirmation != 'Y':
            self.console.print('Aborted.', style='bold red')
            exit()

        # Confirmation dialog if git tag already exists
        git_tag_exists = tag_exists(self.project_version)
        if git_tag_exists:
            confirmation = Prompt.ask(f'The git tag {self.project_version} already exists. Proceed?',
                                      choices=['Y', 'N'])
            if confirmation != 'Y':
                self.console.print('Aborted.', style='bold red')
                exit()

    def make(self) -> None:
        pass

    def get_project_info(self) -> Tuple[str, str]:
        pass

    def check_if_valid(self) -> None:
        pass

    def check_for_changes(self, status: Status):
        status.update(status='Checking for local repo changes...')
        if changes_exist():
            self.console.print('There are local repo changes that have not been commited!', style='bold red')
            exit()

        self.console.log(':white_check_mark: Checked for local repository changes.')

    def docker_build(self, status: Status, arch: str = 'linux/arm64/v8'):
        status.update(status='Building image with Docker...')
        call(self.console, f'docker buildx build -t {self.application_name} --platform {arch} .')
        self.console.log(':white_check_mark: Built Docker image.')

    def docker_push(self, status: Status, versions: list):
        for version in versions:
            image_with_registry = f'{self.get_registry_url()}/{self.project_name}:{version}'

            status.update(status=f'Tagging docker image for {version}...')
            call(self.console, f'docker tag {self.application_name} {image_with_registry}')
            self.console.log(f':white_check_mark: Tagged Docker image for [bold]{version}[/bold]', highlight=False)

            status.update(status=f'Pushing docker image for {version}...')
            call(self.console, f'docker image push {image_with_registry}')
            self.console.log(f':white_check_mark: Pushed Docker image for [bold]{version}[/bold]', highlight=False)

    def git_push(self, status: Status) -> None:
        status.update(status='Pushing changes to git repository...')
        call(self.console, f'git push')
        self.console.log(':white_check_mark: Pushed changes to git repository.')

    def print_build_results(self, versions: list):
        self.console.line(1)
        self.console.print('[bold]Build process complete![/bold]', style='bold green')

        for version in versions:
            latest_image_uri = f'{self.get_registry_url()}/{self.project_name}:{version}'
            self.console.log(f'[bold]Image:[/bold] {latest_image_uri}', highlight=False)

    @property
    def application_name(self) -> str:
        return f'{self.project_name}:{self.project_version}'



import subprocess


def changes_exist() -> bool:
    git_status = subprocess.check_output(['git', 'status', '-s'])
    return len(git_status) > 0


def tag_exists(tag_name: str) -> bool:
    try:
        git_tag = subprocess.check_output(f'git tag | grep {tag_name}', shell=True)
        return len(git_tag) > 0
    except subprocess.CalledProcessError:
        return False


def push() -> bool:
    git_push = subprocess.run(['git', 'push'])
    return git_push.returncode == 0



import argparse

from pyfiglet import Figlet
from rich import get_console

from .platform_angular import Angular
from .platform_mvn import Maven
from .platform_python import Python
from .platform_pypi import Pypi
from .version import __version__


def parse_arguments() -> argparse.Namespace:
    parser = argparse.ArgumentParser(description='local-ci arguments')
    parser.add_argument('platform', type=str, help='Platform for the build process (e.g. mvn, npm, python)')
    return parser.parse_args()


def main():
    args = parse_arguments()

    f = Figlet(font='slant')
    print(f.renderText('local-ci'))

    console = get_console()
    console.print(f'Welcome to [bold]local-ci v{__version__}[/bold]', highlight=False, style='cyan')
    console.print(f'For Python & Angular projects, make sure you are using the desired nvm and pyenv!',
                  highlight=False, style='cyan')
    console.line(1)

    if args.platform == 'mvn':
        platform_builder = Maven()
        platform_builder.make()
    elif args.platform == 'python':
        platform_builder = Python()
        platform_builder.make()
    elif args.platform == 'pypi':
        platform_builder = Pypi()
        platform_builder.make()
    elif args.platform == 'angular':
        platform_builder = Angular()
        platform_builder.make()
    else:
        console.print('Invalid platform for the build process.')


if __name__ == '__main__':
    main()



from typing import Tuple

from .cli_utils import get
from .platform_interface import Platform


class Python(Platform):
    def make(self) -> None:
        with self.console.status("[bold green] Building Maven project...") as status:
            self.check_for_changes(status)

            # Run Python tests
            # status.update(status='Running pytest...')
            # if call('pytest') != 0:
            #     self.console.print("There has been an error with [bold]pytest[/bold]", style="bold red")
            #     exit()

            self.docker_build(status)
            self.docker_push(status, ['latest', self.project_version])
            self.git_push(status)

            status.stop()

        self.print_build_results(['latest', self.project_version])

    def get_project_info(self) -> Tuple[str, str]:
        project_name = get('python setup.py --name')
        project_version = get('python setup.py --version')
        return project_name, project_version



import subprocess

from rich.console import Console


def get(command: str) -> str:
    subprocess_result = subprocess.run([command], capture_output=True, text=True, shell=True)
    return str(subprocess_result.stdout).strip()


def call(console: Console, command: str) -> None:
    try:
        subprocess.run([command], capture_output=True, text=True, shell=True, check=True)
    except subprocess.CalledProcessError as exception:
        console.print(exception.stdout)
        console.print(exception.stderr)

        console.print(f'There has been an error with [bold]{command}[/bold], aborting...', highlight=False)
        exit()



import os
from typing import Tuple

from .cli_utils import get, call
from .platform_interface import Platform


class Maven(Platform):
    def check_if_valid(self) -> None:
        if not os.path.isfile('pom.xml'):
            self.console.print('Invalid Maven project in this directory.', style='bold red')
            exit()

    def make(self) -> None:
        with self.console.status("[bold green] Building Maven project...") as status:
            self.check_for_changes(status)

            # Build with maven
            status.update(status='Building application package with Maven...')
            call(self.console, 'mvn clean package')

            self.console.log(':white_check_mark: Built application package with Maven.')

            self.docker_build(status)
            self.docker_push(status, ['latest', self.project_version])
            self.git_push(status)
            status.stop()

        self.print_build_results(['latest', self.project_version])

    def get_project_info(self) -> Tuple[str, str]:
        project_name = get('mvn help:evaluate -Dexpression=project.name -q -DforceStdout')
        project_version = get('mvn help:evaluate -Dexpression=project.version -q -DforceStdout')
        return project_name, project_version



import os
from typing import Tuple

from .cli_utils import call
from .cli_utils import get
from .platform_interface import Platform


class Pypi(Platform):
    def login_dialog(self) -> None:
        self.username = self.console.input('Enter your [bold]pypi.org[/bold] username: ')
        self.password = self.console.input('Enter your [bold]pypi.org[/bold] password: ', password=True)

        os.putenv('TWINE_USERNAME', self.username)
        os.putenv('TWINE_PASSWORD', self.password)

    def make(self) -> None:
        with self.console.status("[bold green] Building pypi package...") as status:
            self.check_for_changes(status)

            # Build Python package
            status.update(status='Building pypi package...')
            call(self.console, 'python setup.py sdist bdist_wheel')
            call(self.console, 'twine check dist/*')
            self.console.log(':white_check_mark: Built pypi package!')

            status.update(status='Uploading to pypi...')
            call(self.console, f'twine upload --non-interactive dist/*')
            self.console.log(':white_check_mark: Uploaded package to pypi!')

            self.git_push(status)

            status.stop()

        self.print_build_results([self.project_version])

    def print_build_results(self, versions: list):
        self.console.line(1)
        self.console.print('[bold]Build process complete![/bold]', style='bold green')

        for _ in versions:
            self.console.log(f'[bold]Package:[/bold] {self.project_name}==={self.project_version}', highlight=False)

    def get_project_info(self) -> Tuple[str, str]:
        project_name = get('python setup.py --name')
        project_version = get('python setup.py --version')
        return project_name, project_version



from typing import Tuple

from .cli_utils import call
from .cli_utils import get
from .platform_interface import Platform


class Angular(Platform):
    def make(self) -> None:
        with self.console.status("[bold green] Building Angular project...") as status:
            self.check_for_changes(status)

            # Build with Angular
            status.update(status='Installing npm dependencies...')
            call(self.console, 'npm install')
            self.console.log(':white_check_mark: Installed npm dependencies.')

            status.update(status='Building Angular application...')
            call(self.console, 'ng build --outputPath=./build/dist')
            self.console.log(':white_check_mark: Built application with Angular.')

            self.docker_build(status)
            self.docker_push(status, ['latest', self.project_version])
            self.git_push(status)

            status.stop()

        self.print_build_results(['latest', self.project_version])

    def get_project_info(self) -> Tuple[str, str]:
        project_name = get('node -e "console.log(require(\'./package.json\').name)"')
        project_version = get('node -e "console.log(require(\'./package.json\').version)"')
        return project_name, project_version



import pathlib

from setuptools import setup
from localci.version import __version__

HERE = pathlib.Path(__file__).parent
README = (HERE / 'readme.md').read_text()

setup(
    name='localci',
    description='Makes local CI with Python, npm, and Java easy. Integrated with custom docker registry.',
    version=__version__,
    long_description=README,
    long_description_content_type="text/markdown",
    license='MIT',
    author='briccardo',
    author_email='rbiagini02@gmail.com',
    packages=['localci'],
    install_requires=[
        'rich', 'pyfiglet'
    ],
    python_requires='>=3.8',
    entry_points='''
        [console_scripts]
        localci=localci.__main__:main
    '''
)
